/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_GEOMETRY_H__
#define __KIBA_GEOMETRY_H__

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>
#include <glib-object.h>

#ifndef __KIBA_H__
  typedef struct _KibaGeometry		KibaGeometry;
#endif
  typedef struct _KibaGeometryClass	KibaGeometryClass;
  typedef struct _KibaGeometryOptions	KibaGeometryOptions;

#define KIBA_TYPE_GEOMETRY		(kiba_geometry_get_type ())
#define KIBA_GEOMETRY(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_GEOMETRY, KibaGeometry))
#define KIBA_GEOMETRY_CLASS(obj)	(G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_GEOMETRY,  KibaGeometryClass))
#define KIBA_IS_GEOMETRY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_GEOMETRY))
#define KIBA_IS_GEOMETRY_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_GEOMETRY))
#define KIBA_GEOMETRY_GET_CLASS		(G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_GEOMETRY, KibaGeometryClass))

#define DEFAULT_ICON_SIZE		(40)

typedef enum {
	KIBA_POSITION_LEFT_EDGE_TOP,
	KIBA_POSITION_LEFT_EDGE_CENTER,
	KIBA_POSITION_LEFT_EDGE_BOTTOM,
	
	KIBA_POSITION_RIGHT_EDGE_TOP,
	KIBA_POSITION_RIGHT_EDGE_CENTER,
	KIBA_POSITION_RIGHT_EDGE_BOTTOM,
	
	KIBA_POSITION_TOP_EDGE_LEFT,
	KIBA_POSITION_TOP_EDGE_CENTER,
	KIBA_POSITION_TOP_EDGE_RIGHT,
	
	KIBA_POSITION_BOTTOM_EDGE_LEFT,
	KIBA_POSITION_BOTTOM_EDGE_CENTER,
	KIBA_POSITION_BOTTOM_EDGE_RIGHT
} KibaPosition;

typedef struct {
	const char      *name;
	GtkOrientation  orientation;
	gint            edge;
	gint            halignment;
	gint            valignment;
	GdkPoint        anchor;
} KibaLayoutData;

static const KibaLayoutData kiba_layout_data[] = {
	{ "left_edge_top",      GTK_ORIENTATION_VERTICAL,   0, 0, 0, { 0, 1 } },
	{ "left_edge_center",   GTK_ORIENTATION_VERTICAL,   0, 0, 1, { 0, 2 } },
	{ "left_edge_bottom",   GTK_ORIENTATION_VERTICAL,   0, 0, 2, { 0, 3 } },
	{ "right_edge_top",     GTK_ORIENTATION_VERTICAL,   1, 2, 0, { 4, 1 } },
	{ "right_edge_center",  GTK_ORIENTATION_VERTICAL,   1, 2, 1, { 4, 2 } },
	{ "right_edge_bottom",  GTK_ORIENTATION_VERTICAL,   1, 2, 2, { 4, 3 } },
	{ "top_edge_left",      GTK_ORIENTATION_HORIZONTAL, 2, 0, 0, { 1, 0 } },
	{ "top_edge_center",    GTK_ORIENTATION_HORIZONTAL, 2, 1, 0, { 2, 0 } },
	{ "top_edge_right",     GTK_ORIENTATION_HORIZONTAL, 2, 2, 0, { 3, 0 } },
	{ "bottom_edge_left",   GTK_ORIENTATION_HORIZONTAL, 3, 0, 2, { 1, 4 } },
	{ "bottom_edge_center", GTK_ORIENTATION_HORIZONTAL, 3, 1, 2, { 2, 4 } },
	{ "bottom_edge_right",  GTK_ORIENTATION_HORIZONTAL, 3, 2, 2, { 3, 4 } }
};

struct _KibaGeometry {
	GObject              parent_instance;
	
	KibaGeometryOptions *options;
	KibaPosition        position;
	KibaLayoutData      *d;
	GdkRectangle        screen_geometry;	/* monitor/screen geometry */
	GdkRectangle        window_geometry;	/* geometry of the window */
	
	GdkRectangle        last_mouse_position;
	GtkAllocation       allocation;  /* dock-size */
	GtkAllocation       dest_allocation;    //dock-size
	GtkAllocation       last_allocation;    //dock-size before allocation
	int                 drag_distance;       /* distance of object dragging */
	int                 dx, dy;
	int                 hide_requests;
};

struct _KibaGeometryOptions {
	gchar    *position;
	gint     monitor_number;
	double   object_space;
	double   object_margin;
	double   plugin_space;
	double   seperator_size;
};

struct _KibaGeometryClass {
	GObjectClass parent_class;
	
	void (* position_changed) (KibaGeometry *geometry);
	void (* geometry_changed) (KibaGeometry *geometry);
};

/**
 * Creates and returns a new KibaGeometry.
 *
 * @return The KibaGeometry object created.
 */
KibaGeometry* kiba_geometry_new (void);

/**
 * Update Window Geometry.
 *
 * @param geometry The KibaGeometry object.
 */
void kiba_correct_placement (KibaGeometry *geometry);

/**
 * Update Dock Geometry.
 *
 * @param geometry The KibaGeometry object.
 */
void kiba_layout (KibaGeometry *geometry);

/**
 * Set Kiba Position from String.
 *
 * @param geometry The KibaGeometry object.
 * @param string New Position.
 */
void set_position_from_string (KibaGeometry *geometry,
                               const char   *string);

/**
 * Set Kiba Position from KibaPosition.
 *
 * @param geometry The KibaGeometry object.
 * @param position New KibaPosition.
 */
void kiba_set_position (KibaGeometry *geometry,
                        KibaPosition position);

/**
 * Find closest Position.
 *
 * @param geometry The KibaGeometry object.
 * @param x The Mouse X-Position.
 * @param y The Mouse Y-Position.
 * @return The new Position.
 */
int kiba_find_closest_position (KibaGeometry *geometry,
                                int x,
                                int y);

/**
 * Calculate Dock animate Geometry.
 *
 * @param geometry The KibaGeometry object.
 * @param zooming @c TRUE if auto_hide is enabled, otherwise.
 *                @c FALSE.
 * @return @c TRUE if , otherwise
 *         @c FALSe.
 */
gboolean kiba_animate_position_changes (KibaGeometry *geometry,
                                        gboolean     zooming);

/**
 * Calculate Dock animate Geometry.
 *
 * @param geometry The KibaGeometry object.
 * @param stepsize The Animation Stepsize.
 * @param hide @c TRUE if auto_hide is enabled, otherwise.
 *             @c FALSE.
 */
void kiba_calculate_geometry (KibaGeometry  *geometry,
                              int           stepsize,
                              gboolean      hide);

#endif
