/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#include "../config.h"

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>
#include <pango/pango.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif

#ifndef __KIBA_H__
#ifndef __KIBA_PLUGIN_H__
  typedef struct _KibaObject         KibaObject;
#endif
#ifndef __KIBA_AKAMARU_H__
  typedef struct _KibaAkamaruObject  KibaAkamaruObject;
#endif
#endif
  typedef struct _KibaObjectClass    KibaObjectClass;

#define KIBA_TYPE_OBJECT		(kiba_object_get_type ())
#define KIBA_OBJECT(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_OBJECT, KibaObject))
#define KIBA_OBJECT_CLASS(obj)		(G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_OBJECT,  KibaObjectClass))
#define KIBA_IS_OBJECT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_IS_OBJECT_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_OBJECT_GET_CLASS		(G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_OBJECT, KibaObjectClass))

#ifndef __KIBA_H__
	#define SET_FLAG(s,x)			(s |= (1 << x))
	#define DEL_FLAG(s,x)			(s = s & ~(1<<x))
	#define GET_FLAG(s,x)			(s & (1<<x))
#endif

#define FINGER_STEP			0.01

enum {
	KIBA_OBJECT_PRELIGHT = 0,
	KIBA_OBJECT_TITEL,
	KIBA_OBJECT_PULSE,
	KIBA_OBJECT_URGENT,
	KIBA_OBJECT_RECTANGLE,
	KIBA_OBJECT_PROGRESS,
	KIBA_OBJECT_BUBBLE,
	KIBA_OBJECT_DRAGGING,
};

/* this struct saves a kiba objects, like a launcher, clock... */
struct _KibaObject {
	GObject    parent;
	
	KibaAkamaruObject  *akamaru;
	int                flags;	/* rendering flags */
	
	/* used when KIBA_OBJECT_PROGRESS flag is set */
	struct KibaObjectProgress {
		char    *message;       /* message for the bubble */
		double   value;         /* current value of the process */
		double   dest_value;    /* dest value of progress */
		double   fraction;      /* drawn fraction */
	}progress;
	
	/* used when KIBA_OBJECT_BUBBLE flag is set */
	struct KibaObjectBubble {
		PangoLayout  *layout;
		char         *message;
		char         *last_message;
	}bubble;
	
	gboolean                  at_dock;
	gboolean                  exec;       /* true on exec */
	char                     *name;       /* Titel for pango rendering */
	char                     *exec_command;
	int                       exec_pid;
	int                       position;   /* position of object in plugin (1=leftmost for horizontal docks) */
	GdkRectangle              geometry;   /* saves width/height x/y of the object, when the object is in motion,
						this is the aim geometry */
	GdkRectangle              animate;    /* saves width/height x/y of the object, when the object is in motion,
						this is the dynamic geometry */
	
	GdkPixbuf                *bg_pixbuf;
	GdkPixbuf                *fg_pixbuf;
#ifdef HAVE_SVG
	RsvgHandle               *bg_svg_handle;
	RsvgHandle               *fg_svg_handle;
#endif
	cairo_surface_t          *background_surface;
	cairo_surface_t          *foreground_surface;
	cairo_surface_t          *text_surface;
	gboolean                  bgsurface_needs_update;
	gboolean                  fgsurface_needs_update;
	gboolean                  text_surface_needs_update;
	GdkRectangle              text_buffer_size;
	PangoLayout              *pango_layout;       //saves the pango layout
	
	gboolean                  rerender;
	double                    alpha;
	
	char                     *title;
	
	/* pulse effect */
	double                    pulse_size;  /* scaled pulse size */
	
	/* geometry */
	double                    drawn_rotate_angle;
	double                    rotate_angle;
	double                    scale_factor; /* scale factor for rotation */
	double                    normal_size;  /* unscalled size */
	double                    size;         /* scalled size */
	double                    zoom_size;    /* used to avoid to big stepsizes */
};

struct _KibaObjectClass {
	GObjectClass	parent_class;
	
	void (* button_released)  (KibaObject *object, gint button);
	void (* scroll_event)     (KibaObject *object, gint direction);
	void (* drag_event)       (KibaObject *object, GdkDragContext *context, GtkSelectionData *data);
};

/**
 * Get the Object List.
 *
 * @return The GList of all initialled objects.
 */
GList* kiba_objects_get ();

/**
 * Creates and returns a new KibaObject.
 *
 * @param settings The KibaSettings struct.
 * @param object_name The Name of the object.
 * @param width The Object Width.
 * @param height The Object Height.
 * @return The KibaObject object created.
 */
KibaObject* kiba_object_new (const char  *object_name,
                             double       width,
                             double       height);

/**
 * Get the Object under the Mouse Pointer.
 *
 * @param window The Kiba Window.
 * @param geometry The KibaGeometry struct.
 * @return The found KibaObject or NULL.
 */
KibaObject* kiba_object_get_under_pointer (GdkWindow     *window,
                                           KibaGeometry  *geometry);

/**
 * Get the Object with the biggest size.
 *
 * @return The found KibaObject or NULL.
 */
KibaObject* kiba_object_get_biggest (void);

/**
 * Free/Destroy a Object.
 *
 * @param object The KibaObject to free.
 */
void kiba_object_free (KibaObject *object);

/**
 * Launch a Object Command (timeout).
 *
 * @param data The KibaObject to exec.
 * @return @c TRUE as long as it is permitted by Flags to execute the Command, otherwise
 *         @c FALSE when no Command found, or the Command was launched.
 */
gint kiba_object_exec (gpointer data);

/**
 * Hide all initialled Objects.
 *
 * @param geometry The KibaGeometry struct.
 */
void kiba_objects_hide (KibaGeometry *geometry);

/**
 * Unset Prelight for a Object.
 *
 * @param object The KibaObject to unset the Prelight.
 */
void kiba_object_unset_prelight (KibaObject *object);

/**
 * Update the Geometry for a object.
 *
 * @param object The KibaObject.
 * @param stepsize The maximal Stepsize for The geometry step.
 */
void kiba_object_calculate_geometry (KibaObject  *object,
                                     int         stepsize);

/**
 * Bounce a Object.
 *
 * @param object The KibaObject to bounce.
 * @param geometry The KibaGeometry struct.
 * @param bounce_height The Height for the Bounce.
 */
void kiba_object_bounce (KibaObject    *object,
                         KibaGeometry  *geometry,
                         double         bounce_height,
                         gboolean       physics);

/**
 * Render a Object Surface Buffer.
 *
 * @param object The KibaObject to update the Surface.
 * @param drawing_context The Cairo context to render on.
 * @param background @c TRUE if the background Surface Buffer should be drawn, otherwise
 *                   @c FALSE.
 */
void kiba_object_draw_surface_buffer (KibaObject *object,
                                      cairo_t    *drawing_context,
                                      gboolean   background);

/**
 * Set the _NET_WM_ICON_GEOMETRY proberty on a Window for a Object.
 *
 * @param object The KibaObject to set the proberty to.
 * @param window The GdkWindow to set the proberty on.
 * @param geometry The KibaGeometry struct.
 */
void kiba_object_set_icon_geometry (KibaObject   *object,
                                    GdkWindow    *window,
                                    KibaGeometry *geometry);

#endif
