/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_RENDER_H__
#define __KIBA_RENDER_H__

#include "../config.h"
#ifdef HAVE_GLITZ
#ifdef CAIRO_HAS_GLITZ_SURFACE
  #define USE_GLITZ
#endif
#endif

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif
#ifdef USE_GLITZ
  #include <glitz-glx.h>
  #include <cairo-glitz.h>
#endif

/**
 * Draw Shape on Context.
 */
void kiba_paint_shape (KibaGeometry  *geometry,
                       cairo_t       *draw_context,
                       const char    *topleft_corner,
                       const char    *topright_corner,
                       const char    *bottomleft_corner,
                       const char    *bottomright_corner,
                       gboolean       dock_shape,
                       gboolean       draw_all_border,
                       double         x0,
                       double         y0,
                       double         x1,
                       double         y1,
                       double         radius,
                       double         border_width);

/**
 * Get Gradient Pattern.
 */
cairo_pattern_t* kiba_get_gradient_pattern (KibaGeometry *geometry,
                                            const char   *rgba_1,
                                            const char   *rgba_2,
                                            const char   *rgba_3,
                                            double        x0,
                                            double        x1,
                                            double        y0,
                                            double        y1);

/**
 * Get Pattern for PNG File.
 *
 * @param cr The Cairo Context.
 * @param png_file The PNG File.
 * @param rotate.
 * @return The cairo_pattern_t created.
 */
cairo_pattern_t* kiba_get_png_pattern (cairo_t    *cr,
                                       const char *png_file,
                                       gboolean    rotate);

/**
 * Get Linear/Radial Pattern.
 */
cairo_pattern_t* kiba_get_bg_pattern (KibaGeometry *geometry,
                                      gboolean      linear,
                                      const char   *rgba_1,
                                      const char   *rgba_2,
                                      const char   *rgba_3,
                                      double        affection,
                                      double        step_size,
                                      double        radius_1,
                                      double        radius_2);

/**
 * Load PNG File at Path in the Pixbuf.
 *
 * @param icon The PNG Icon Name, or the Absolute Path.
 * @param size The Pixbuf size.
 * @param pixbuf The Pixbuf.
 * @return @c TRUE on success, otherwise
 *         @c FALSE.
 */
gboolean kiba_load_png (const char *icon,
                        int        size,
                        GdkPixbuf  **pixbuf);

#ifdef HAVE_SVG
/**
 * Load SVG File at Path in the RsvgHandle.
 *
 * @param icon The SVG Icon Name, or the Absolute Path.
 * @param size The SVG size.
 * @param pixbuf The RsvgHandle.
 * @return @c TRUE on success, otherwise
 *         @c FALSE.
 */
gboolean kiba_load_svg (const char  *icon,
                        double      *size,
                        RsvgHandle **handle);
#endif

/**
 * Switch pointer of Surfaces.
 *
 * @param flip_surface A cairo_surface_t.
 * @param flop_surface A cairo_surface_t.
 */
void kiba_switch_surfaces (cairo_surface_t **flip_surface,
                           cairo_surface_t **flop_surface);

/**
 * Get Red, Green, Blue double Values from String.
 *
 * @param html_color The RGB-String.
 * @param r The Red Value.
 * @param g The Green Value.
 * @param b The Blue Value.
 */
void kiba_get_rgb_from_string (const char *html_color,
                               double     *r,
                               double     *g,
                               double     *b);

/**
 * Get Red, Green, Blue, Alpha double Values from String.
 *
 * @param html_color The RGBA-String.
 * @param r The Red Value.
 * @param g The Green Value.
 * @param b The Blue Value.
 * @param alpha The Alpha Value.
 */
void kiba_get_rgba_from_string (const char *html_color,
                                double     *r,
                                double     *g,
                                double     *b,
                                double     *alpha);

#ifdef USE_GLITZ
/**
 * Create Glitz Drawable on Widget.
 *
 * @param widget The GtkWidget to create the Drawable on.
 * @param format glitz_format_t.
 * @param drawable The glitz_drawable_t.
 * @param draw_format The glitz_drawable_format_t.
 * @param width The Drawable width.
 * @param height The Drawable height.
 */
void kiba_create_glitz_drawable (GtkWidget                *widget,
                                 glitz_format_t          **format,
                                 glitz_drawable_t        **drawable,
                                 glitz_drawable_format_t  *draw_format,
                                 int                        width,
                                 int                        height);

/**
 * Get Glitz Draw Format.
 *
 * @param glitz_format The glitz_drawable_format_t.
 */
void kiba_find_glitz_draw_format (glitz_drawable_format_t **glitz_format);

/**
 * Get Glitz Colormap/Visual.
 *
 * @param draw_format The glitz_drawable_format_t.
 * @param visual The returned GdkVisual.
 * @param colormap The returned GdkColormap.
 */
void kiba_get_glitz_colormap (glitz_drawable_format_t *draw_format,
                              GdkVisual              **visual,
                              GdkColormap            **colormap);
#endif

/**
 * Rerender Kiba.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_render (GtkWidget *kiba_widget);

/**
 * Clear the Kiba Window.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_clear_window (GtkWidget *kiba_widget);

/**
 * Update Kiba Surface Bufers.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_update_surface_buffers (GtkWidget *kiba_widget);

/**
 * Update Kiba Surface Bufers.
 *
 * @param kiba_widget The kiba Widget.
 * @return The cairo_t created.
 */
cairo_t* kiba_cairo_create (GtkWidget *kiba_widget);


#endif
